/**
 * https://docs.python.org/ja/3/extending/extending.html

 g++ -g -DPIC -shared -fpic -o sample1.so sample1.cc -I/usr/local/include/python3.6m
 python3.6m -c "import sample1; print(sample1.dummy())"

 g++ -g -DPIC -shared -fpic -o sample1.so sample1.cc -I/usr/local/python375/include/python3.7dm
 /usr/local/python375/bin/python3.7dm -c "import sample1; print(sample1.dummy())"
*/

#define PY_SSIZE_T_CLEAN
#include <Python.h>

class something {
public:
  PyObject *somelist; // Python list object
  something() {
    printf("something called\n");
    somelist = PyList_New(1);
    PyList_SET_ITEM(somelist, 0, PyLong_FromLong(1L));
  }
  ~something() {
    printf("~something called\n");
    //PyThreadState *_tstate = PyThreadState_GET();
    //printf("_state = %p\n", _tstate);
    Py_DECREF(somelist);
  }
};

static something obj; // Global static object

static PyObject *
dummy(PyObject *self, PyObject *args)
{
  Py_INCREF(obj.somelist);
  return obj.somelist;
}

static PyMethodDef Sample1Methods[] = {
    {"dummy",  dummy, METH_VARARGS, "dummy function"},
    {NULL, NULL, 0, NULL}
};

static struct PyModuleDef sample1module = {
    PyModuleDef_HEAD_INIT,
    "sample1",
    NULL,
    -1,
    Sample1Methods
};

extern "C"
PyMODINIT_FUNC
PyInit_sample1(void)
{
    return PyModule_Create(&sample1module);
}
